package vishnu.cluster;

import java.util.*;


public class ClusterArray
{
    Vector contents;
    
    ClusterArray(int num_of_clusters)
    {
	contents = new Vector(num_of_clusters);
	
	for(int i = 0; i < num_of_clusters; i++)
	    contents.addElement(new Cluster(i, null, null, null, 0, 1));
    }
    
    
    // counts the number of clusters that are less dissimilar than disim
    private void sweep_tree (Cluster cluster, double disim, int min_c_items, int[] clusters)
    {
	cluster.split = 0;
	
	if (cluster.id == -1){
	    if (cluster.children_sim <= disim && cluster.items >= min_c_items){
		cluster.split = 1;
		contents.addElement(cluster);
		clusters[0]++;
	    }else{
		sweep_tree (cluster.child_1, disim, min_c_items, clusters);
		sweep_tree (cluster.child_2, disim, min_c_items, clusters);
	    }
    	}else{
	    if (cluster.items >= min_c_items){
		cluster.split = 1;
		contents.addElement(cluster);
		clusters[0]++;
	    }
	}
    }


    double split (Cluster cluster, double[] heights, int n, int target, int min_c_items, int[] clusters)
    {
	
	/*
	  Try to split 'cluster', made up of 'n' vectors, into 'target'
	  clusters by incrementing iteratively the mimimum required
	  similarity between the 'cluster\'s' children. The result is an
	  array of clusters, 'cluster_ary' which contains 'clusters'
	  clusters.  Obviously the final number of clusters may not be the
	  same as the desired number of clusters. The function return the
	  threshold similarity which resulted in the clusters being split
	  in 'clusters' clusters.
	*/
	
	int i;
	
	if (target < 2) target = 2;
	
	if (target > n) target = n;
	
	// height store minimum similarity values
	// this is then the iterative incrementing loop
	for (i = target - 2; i <= n - 1; i++){
	    clusters[0] = 0;
	    contents.removeAllElements();
	    sweep_tree (cluster, heights[i], min_c_items, clusters);
	    
	    if (clusters[0] >= target) break;
	}
	
	// if target - 2 > n - 1, i.e. if the previous loop has not been 
	// entered at all
	if (i > n - 1){
	    /* ignore the min_c_size requirement */
	    i = target - 1;
	    clusters[0] = 0;
	    contents.removeAllElements();
	    sweep_tree (cluster, heights[i], 1, clusters);
	}
	
	return heights[i];
    }
}
